@@ -3,7 +3,7 @@ import json
 import os
 import warnings
 from dataclasses import dataclass
-from typing import Optional
+from typing import Optional, Union
 
 import numpy as np
 
@@ -45,9 +45,10 @@ TILEDB_KWARG_DEFAULTS = {
     "index_dims": None,
     "allows_duplicates": True,
     "mode": "ingest",
-    "attr_filters": None,
-    "dim_filters": None,
-    "coords_filters": None,
+    "attr_filters": True,
+    "dim_filters": True,
+    "coords_filters": True,
+    "offsets_filters": True,
     "full_domain": False,
     "tile": None,
     "row_start_idx": None,
@@ -130,12 +131,7 @@ class ColumnInfo:
                 # see https://github.com/pandas-dev/pandas/issues/40574
                 nullable = bool(dtype.construct_array_type()._can_hold_na)
 
-            return cls(
-                np.dtype(np_type),
-                repr=dtype.name,
-                nullable=nullable,
-                var=var,
-            )
+            return cls(np.dtype(np_type), repr=dtype.name, nullable=nullable, var=var)
 
         # bool type
         if pd_types.is_bool_dtype(dtype):
@@ -165,7 +161,8 @@ class ColumnInfo:
         # string types
         # don't use pd_types.is_string_dtype() because it includes object types too
         if dtype.type in (np.bytes_, np.str_):
-            return cls(dtype)
+            # str and bytes are always stored as var-length
+            return cls(dtype, var=True)
 
         raise NotImplementedError(f"{dtype} dtype not supported")
 
@@ -180,17 +177,34 @@ def _get_column_infos(df, column_types, varlen_types):
     return column_infos
 
 
+def _get_schema_filters(filters):
+    if filters is True:
+        # default case, unspecified: use libtiledb defaults
+        return None
+    elif filters is None:
+        # empty filter list (schema uses zstd by default if unspecified)
+        return tiledb.FilterList()
+    elif isinstance(filters, (list, tiledb.FilterList)):
+        return tiledb.FilterList(filters)
+    elif isinstance(filters, tiledb.libtiledb.Filter):
+        return tiledb.FilterList([filters])
+    else:
+        raise ValueError("Unknown FilterList type!")
+
+
+def _get_attr_dim_filters(name, filters):
+    if isinstance(filters, dict):
+        # support passing a dict of filters per-attribute
+        return _get_schema_filters(filters.get(name, True))
+    else:
+        return _get_schema_filters(filters)
+
+
 def _get_attrs(names, column_infos, attr_filters, ctx=None):
     attrs = []
     attr_reprs = {}
     for name in names:
-        if isinstance(attr_filters, dict):
-            filters = attr_filters.get(name)
-        elif attr_filters is not None:
-            filters = attr_filters
-        else:
-            filters = tiledb.FilterList([tiledb.ZstdFilter(1, ctx=ctx)], ctx=ctx)
-
+        filters = _get_attr_dim_filters(name, attr_filters)
         column_info = column_infos[name]
         attrs.append(
             tiledb.Attr(
@@ -233,30 +247,23 @@ def dim_for_column(
     dtype = dim_info.dtype
 
     if full_domain:
-        if not dim_info.dtype in (np.bytes_, np.unicode_):
+        if dim_info.dtype not in (np.bytes_, np.unicode_):
             # Use the full type domain, deferring to the constructor
-            (dtype_min, dtype_max) = tiledb.libtiledb.dtype_range(dim_info.dtype)
+            dtype_min, dtype_max = tiledb.libtiledb.dtype_range(dim_info.dtype)
 
             dim_max = dtype_max
             if dtype.kind == "M":
                 date_unit = np.datetime_data(dtype)[0]
-                dim_min = np.datetime64(dtype_min + 1, date_unit)
+                dim_min = np.datetime64(dtype_min, date_unit)
                 tile_max = np.iinfo(np.uint64).max - tile
-
-                # modular arithmetic gives misleading overflow warning.
-                with np.errstate(over="ignore"):
-                    dtype_range = np.uint64(dtype_max) - np.uint64(dtype_min)
-
-                if np.abs(dtype_range) > tile_max:
+                if np.uint64(dtype_max - dtype_min) > tile_max:
                     dim_max = np.datetime64(dtype_max - tile, date_unit)
-            elif dtype is np.int64:
-                dim_min = dtype_min + 1
             else:
                 dim_min = dtype_min
 
             if dtype.kind != "M" and np.issubdtype(dtype, np.integer):
                 tile_max = np.iinfo(np.uint64).max - tile
-                if np.abs(np.uint64(dtype_max) - np.uint64(dtype_min)) > tile_max:
+                if np.uint64(dtype_max - dtype_min) > tile_max:
                     dim_max = dtype_max - tile
         else:
             dim_min, dim_max = (None, None)
@@ -265,16 +272,15 @@ def dim_for_column(
         dim_min = np.min(col_values)
         dim_max = np.max(col_values)
 
-    if not dim_info.dtype in (np.bytes_, np.unicode_):
+    if dim_info.dtype not in (np.bytes_, np.unicode_):
         if np.issubdtype(dtype, np.integer) or dtype.kind == "M":
-            dim_range = np.uint64(np.abs(np.uint64(dim_max) - np.uint64(dim_min)))
-            # we can't make a tile larger than the dimension range
-            if dim_range < tile:
-                tile = dim_range
-            if tile < 1:
-                tile = 1
-        elif np.issubdtype(dtype, np.float64):
-            dim_range = dim_max - dim_min
+            dim_range = np.uint64(dim_max - dim_min)
+            # we can't make a tile larger than the dimension range or lower than 1
+            tile = max(1, min(tile, dim_range))
+        elif np.issubdtype(dtype, np.floating):
+            # this difference can be inf
+            with np.errstate(over="ignore"):
+                dim_range = dim_max - dim_min
             if dim_range < tile:
                 tile = np.ceil(dim_range)
 
@@ -307,7 +313,13 @@ def get_index_metadata(dataframe):
 
 
 def create_dims(
-    ctx, dataframe, index_dims, tile=None, full_domain=False, sparse=None, filters=None
+    ctx,
+    dataframe,
+    index_dims,
+    tile=None,
+    full_domain=False,
+    sparse=None,
+    filters: Union[dict, "tiledb.FilterList", None] = None,
 ):
     import pandas as pd
 
@@ -375,15 +387,7 @@ def create_dims(
             raise ValueError(f"Empty column '{name}' cannot be used as dimension")
 
         # get the FilterList, if any
-        if isinstance(filters, dict):
-            if name in filters:
-                dim_filters = filters[name]
-            else:
-                dim_filters = None
-        elif filters is not None:
-            dim_filters = filters
-        else:
-            dim_filters = None
+        dim_filters = _get_attr_dim_filters(name, filters)
 
         if per_dim_tile and name in tile:
             dim_tile = tile[name]
@@ -460,7 +464,6 @@ def _df_to_np_arrays(df, column_infos, fillna):
     nullmaps = {}
     for name, column in df.items():
         column_info = column_infos[name]
-
         if fillna is not None and name in fillna:
             column = column.fillna(fillna[name])
 
@@ -520,9 +523,10 @@ def from_pandas(uri, dataframe, **kwargs):
     sparse = tiledb_args["sparse"]
     index_dims = tiledb_args.get("index_dims", None)
     mode = tiledb_args.get("mode", "ingest")
-    attr_filters = tiledb_args.get("attr_filters", None)
-    dim_filters = tiledb_args.get("dim_filters", None)
-    coords_filters = tiledb_args.get("coords_filters", None)
+    attr_filters = tiledb_args.get("attr_filters", True)
+    dim_filters = tiledb_args.get("dim_filters", True)
+    coords_filters = tiledb_args.get("coords_filters", True)
+    offsets_filters = tiledb_args.get("offsets_filters", True)
     full_domain = tiledb_args.get("full_domain", False)
     capacity = tiledb_args.get("capacity", False)
     tile = tiledb_args.get("tile", None)
@@ -567,6 +571,11 @@ def from_pandas(uri, dataframe, **kwargs):
             }
         )
 
+    # handle coords/offsets filters
+    coords_filters = _get_schema_filters(coords_filters)
+    offsets_filters = _get_schema_filters(offsets_filters)
+
+    # create column type <> Dim mappings
     column_infos = _get_column_infos(dataframe, column_types, varlen_types)
 
     if create_array:
@@ -607,6 +616,7 @@ def from_pandas(uri, dataframe, **kwargs):
             cell_order=cell_order,
             tile_order=tile_order,
             coords_filters=coords_filters,
+            offsets_filters=offsets_filters,
             allows_duplicates=allows_duplicates,
             capacity=capacity,
             sparse=sparse,
@@ -647,7 +657,6 @@ def from_pandas(uri, dataframe, **kwargs):
                         coords.append(write_dict.pop(dim_name))
                     else:
                         coords.append(dataframe.index.get_level_values(k))
-
                 # TODO ensure correct col/dim ordering
                 libtiledb._setitem_impl_sparse(A, tuple(coords), write_dict, nullmaps)
 
@@ -660,7 +669,7 @@ def from_pandas(uri, dataframe, **kwargs):
             A.close()
 
 
-def open_dataframe(uri, *, attrs=None, use_arrow=None, ctx=None):
+def open_dataframe(uri, *, attrs=None, use_arrow=None, idx=slice(None), ctx=None):
     """Open TileDB array at given URI as a Pandas dataframe
 
     If the array was saved using tiledb.from_pandas, then columns
@@ -681,7 +690,7 @@ def open_dataframe(uri, *, attrs=None, use_arrow=None, ctx=None):
 
     # TODO support `distributed=True` option?
     with tiledb.open(uri, ctx=ctx) as A:
-        df = A.query(attrs=attrs, use_arrow=use_arrow).df[:]
+        df = A.query(attrs=attrs, use_arrow=use_arrow, coords=True).df[idx]
 
     if attrs and list(df.columns) != list(attrs):
         df = df[attrs]
