@@ -1,10 +1,7 @@
-from __future__ import absolute_import
+import pytest
 
-try:
-    import pandas as pd
-    import pandas._testing as tm
-except ImportError:
-    pd = tm = None
+pd = pytest.importorskip("pandas")
+tm = pd._testing
 
 import copy
 import glob
@@ -13,8 +10,7 @@ import random
 from pathlib import Path
 
 import numpy as np
-import pytest
-from numpy.testing import assert_array_equal, temppath
+from numpy.testing import assert_array_equal
 
 import tiledb
 from tiledb.dataframe_ import ColumnInfo
@@ -169,12 +165,10 @@ class TestColumnInfo:
 
     def test_object_dtype(self):
         self.assertColumnInfo(
-            ColumnInfo.from_values(pd.Series(["hello", "world"])),
-            np.dtype("<U"),
+            ColumnInfo.from_values(pd.Series(["hello", "world"])), np.dtype("<U")
         )
         self.assertColumnInfo(
-            ColumnInfo.from_values(pd.Series([b"hello", b"world"])),
-            np.dtype("S"),
+            ColumnInfo.from_values(pd.Series([b"hello", b"world"])), np.dtype("S")
         )
         for s in ["hello", b"world"], ["hello", 1], [b"hello", 1]:
             pytest.raises(NotImplementedError, ColumnInfo.from_values, pd.Series(s))
@@ -243,12 +237,6 @@ class TestDimType:
 
 
 class TestPandasDataFrameRoundtrip(DiskTestCase):
-    def setUp(self):
-        if pd is None:
-            self.skipTest("Pandas not available")
-        else:
-            super(PandasDataFrameRoundtrip, self).setUp()
-
     def test_dataframe_basic_rt1_manual(self):
 
         uri = self.path("dataframe_basic_rt1_manual")
@@ -324,13 +312,23 @@ class TestPandasDataFrameRoundtrip(DiskTestCase):
         uri = self.path("dataframe_basic_rt1_unlimited")
         tiledb.from_pandas(uri, df, full_domain=True, sparse=False, ctx=ctx)
 
+        with tiledb.open(uri) as A:
+            dim = A.schema.domain.dim(0)
+            assert dim.domain[0] == 0
+            assert dim.domain[1] == dtype_max(np.uint64) - dim.tile
+
         for use_arrow in None, False, True:
             df_readback = tiledb.open_dataframe(uri, use_arrow=use_arrow)
             tm.assert_frame_equal(df, df_readback)
 
             attrs = ["s", "q", "t"]
-            df_arrow = tiledb.open_dataframe(uri, attrs=attrs, use_arrow=use_arrow)
-            tm.assert_frame_equal(df[attrs], df_arrow)
+            df_readback = tiledb.open_dataframe(uri, attrs=attrs, use_arrow=use_arrow)
+            tm.assert_frame_equal(df[attrs], df_readback)
+
+            df_readback = tiledb.open_dataframe(
+                uri, idx=slice(2, 4), use_arrow=use_arrow
+            )
+            tm.assert_frame_equal(df[2:5], df_readback)
 
     def test_dataframe_basic2(self):
         uri = self.path("dataframe_basic_rt2")
@@ -534,6 +532,19 @@ class TestPandasDataFrameRoundtrip(DiskTestCase):
         df_bk = tiledb.open_dataframe(uri)
         tm.assert_frame_equal(df_bk, df)
 
+    def test_dataframe_empty(self):
+        dfs = [
+            make_dataframe_basic1(),
+            make_dataframe_basic2(),
+            make_dataframe_basic3(),
+        ]
+        for i, df in enumerate(dfs, start=1):
+            for sparse in False, True:
+                uri = self.path(f"dataframe_empty_{i}_{sparse}")
+                tiledb.from_pandas(uri, df, sparse=sparse)
+                with tiledb.open(uri) as A:
+                    tm.assert_frame_equal(df.iloc[:0], A.df[tiledb.EmptyRange])
+
     def test_csv_dense(self):
         col_size = 10
         df_data = {
@@ -642,7 +653,7 @@ class TestPandasDataFrameRoundtrip(DiskTestCase):
         df.sort_values("time", inplace=True)
         df.to_csv(tmp_csv, index=False)
 
-        attrs_filters = tiledb.FilterList([tiledb.ZstdFilter(1)])
+        attrs_filters = tiledb.FilterList([tiledb.ZstdFilter(3)])
         # from_pandas default is 1, so use 7 here to check
         #   the arg is correctly parsed/passed
         coords_filters = tiledb.FilterList([tiledb.ZstdFilter(7)])
@@ -666,6 +677,7 @@ class TestPandasDataFrameRoundtrip(DiskTestCase):
             sparse=True,
             tile={"time": 5},
             coords_filters=coords_filters,
+            attr_filters=attrs_filters,
         )
 
         t0, t1 = df.time.min(), df.time.max()
@@ -1074,3 +1086,69 @@ class TestPandasDataFrameRoundtrip(DiskTestCase):
             for array1, array2 in zip(df["data"].values, df2["data"].values):
                 self.assertEqual(array1.dtype, array2.dtype)
                 np.testing.assert_array_equal(array1, array2)
+
+
+class TestFromPandasOptions(DiskTestCase):
+    def test_filters_options(self):
+        def assert_filters_eq(left, right):
+            # helper to check equality:
+            # - None should produce empty FilterList
+            # - a dict should match the first and only key/None
+            if isinstance(left, dict):
+                assert len(left) == 1
+                left = list(left.values())[0]
+            if isinstance(right, dict):
+                assert len(right) == 1
+                right = list(right.values())[0]
+
+            left = (
+                tiledb.FilterList([left]) if isinstance(left, tiledb.Filter) else left
+            )
+            right = (
+                tiledb.FilterList([right]) if isinstance(right, tiledb.Filter) else left
+            )
+
+            if left is None:
+                left = tiledb.FilterList()
+            if right is None:
+                right = tiledb.FilterList([right])
+
+            assert left == right
+
+        df_orig = pd.DataFrame({"x": pd.Series([1, 2, 3])})
+        df_orig.index.name = "d"
+
+        filters_to_check = [
+            tiledb.ZstdFilter(2),
+            [tiledb.ZstdFilter(2)],
+            None,
+            tiledb.FilterList(),
+            tiledb.FilterList([tiledb.ZstdFilter(2), tiledb.ZstdFilter(4)]),
+            {"d": None},
+            {"d": tiledb.FilterList([tiledb.ZstdFilter(3), tiledb.ZstdFilter(5)])},
+        ]
+
+        # mapping of options to getters for comparison on read-back
+        checks = [
+            ("dim_filters", lambda A: A.schema.domain.dim(0).filters),
+            ("attr_filters", lambda A: A.schema.attr(0).filters),
+            ("coords_filters", lambda A: A.schema.coords_filters),
+            ("offsets_filters", lambda A: A.schema.offsets_filters),
+        ]
+
+        for opt, getter in checks:
+            df = df_orig
+
+            # change the names for attr test expectation
+            if opt == "attr_filters":
+                df.index.name = ""
+                df = df.rename(columns={"x": "d"})
+
+            for f in filters_to_check:
+                if opt in ("coords_filters", "offsets_filters") and isinstance(f, dict):
+                    continue
+
+                uri = self.path()
+                tiledb.from_pandas(uri, df, **{opt: f})
+                with tiledb.open(uri) as A:
+                    assert_filters_eq(getter(A), f)
